# Generated by Django 3.2.15 on 2022-10-07 07:01

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='NotificationHistory',
            fields=[
                ('channel', models.PositiveSmallIntegerField(choices=[(100, 'push'), (200, 'email'), (300, 'sms'), (400, 'call'), (500, 'chat'), (600, 'slack'), (700, 'webhook'), (1000, 'console')])),
                ('subject', models.CharField(max_length=1024)),
                ('body', models.TextField(blank=True, null=True)),
                ('body_html', models.TextField(blank=True, null=True)),
                ('data', models.TextField(blank=True, null=True)),
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='ID')),
                ('timestamp', models.DateTimeField(auto_now=True)),
                ('users', models.ManyToManyField(help_text='Users this notification was sent to', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name_plural': 'Notification history',
            },
        ),
        migrations.CreateModel(
            name='UserDevice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=255, null=True, verbose_name='Name')),
                ('active', models.BooleanField(default=True, help_text='Inactive devices will not be sent notifications', verbose_name='Is active')),
                ('date_created', models.DateTimeField(auto_now_add=True, null=True, verbose_name='Creation date')),
                ('device_id', models.CharField(blank=True, db_index=True, help_text='Unique device identifier', max_length=255, null=True, verbose_name='Device ID')),
                ('registration_id', models.TextField(verbose_name='Registration token')),
                ('type', models.CharField(choices=[('ios', 'ios'), ('android', 'android'), ('web', 'web')], max_length=10)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'User device',
                'verbose_name_plural': 'User devices',
            },
        ),
        migrations.CreateModel(
            name='NotificationTemplate',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('channel', models.PositiveSmallIntegerField(choices=[(100, 'push'), (200, 'email'), (300, 'sms'), (400, 'call'), (500, 'chat'), (600, 'slack'), (700, 'webhook'), (1000, 'console')])),
                ('subject', models.CharField(max_length=1024)),
                ('body', models.TextField(blank=True, null=True)),
                ('body_html', models.TextField(blank=True, null=True)),
                ('data', models.TextField(blank=True, null=True)),
                ('slug', models.CharField(max_length=255, unique=True)),
                ('history', models.ManyToManyField(blank=True, to='df_notifications.NotificationHistory')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



from df_notifications.decorators import register_notification
from df_notifications.models import NotificationTemplate
from django.contrib.auth.models import User
from django.db import models


class Post(models.Model):
    title = models.CharField(max_length=255)
    description = models.TextField()
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)
    is_published = models.BooleanField(default=False)


@register_notification(model=Post, slugs=["post_created"])
def post_created(prev: Post, next: Post, template: NotificationTemplate, **kwargs):
    context = {
        "prev": prev,
        "instance": next,
    }
    if prev is None:
        template.send(users=list(User.objects.all()), context=context)


@register_notification(model=Post, slugs=["post_published"])
def post_published_async(
    prev: Post, next: Post, template: NotificationTemplate, **kwargs
):
    context = {"prev_title": prev.title}
    if not (prev and prev.is_published) and next.is_published:
        template.send_async(
            users=list(User.objects.all()), instance=next, context=context
        )



from df_notifications.drf.serializers import UserDeviceSerializer
from df_notifications.models import UserDevice
from fcm_django.api.rest_framework import FCMDeviceAuthorizedViewSet


class UserDeviceViewSet(FCMDeviceAuthorizedViewSet):
    queryset = UserDevice.objects.all()
    serializer_class = UserDeviceSerializer



DEBUG = True

ROOT_URLCONF = "tests.urls"
SECRET_KEY = "111111"

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "rest_framework",
    "fcm_django",
    "df_notifications",
    "django_slack",
    "tests.test_app.apps.TestAppConfig",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": "db.sqlite3",
    }
}

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
        },
    },
    "root": {
        "handlers": ["console"],
        "level": "INFO",
    },
}

STATIC_URL = "/static/"

ALLOWED_HOSTS = ["*"]

CELERY_BROKER_URL = "redis://127.0.0.1:6379/1"
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#std:setting-result_backend
CELERY_RESULT_BACKEND = CELERY_BROKER_URL



from celery import current_app as app
from django.apps import apps
from django.conf import settings
from django.contrib.auth import get_user_model
from django.core.mail import EmailMultiAlternatives
from django.db import models
from django.template import Context
from django.template import Template
from django.utils.translation import gettext_lazy as _
from django_slack import slack_message
from fcm_django.models import AbstractFCMDevice
from firebase_admin.messaging import Message
from firebase_admin.messaging import Notification

import json
import logging
import requests


class NotificationChannels(models.IntegerChoices):
    PUSH = 100, "push"
    EMAIL = 200, "email"
    SMS = 300, "sms"
    CALL = 400, "call"
    CHAT = 500, "chat"
    SLACK = 600, "slack"
    WEBHOOK = 700, "webhook"

    CONSOLE = 1000, "console"


class AbstractNotificationBase(models.Model):
    channel = models.PositiveSmallIntegerField(choices=NotificationChannels.choices)
    subject = models.CharField(max_length=1024)
    body = models.TextField(null=True, blank=True)
    body_html = models.TextField(null=True, blank=True)
    data = models.TextField(null=True, blank=True)

    class Meta:
        abstract = True


class NotificationHistory(AbstractNotificationBase):
    id = models.BigAutoField(
        verbose_name=_("ID"),
        primary_key=True,
    )
    timestamp = models.DateTimeField(auto_now=True)
    users = models.ManyToManyField(
        settings.AUTH_USER_MODEL, help_text="Users this notification was sent to"
    )

    class Meta:
        verbose_name_plural = "Notification history"


class NotificationTemplate(AbstractNotificationBase):
    history = models.ManyToManyField(NotificationHistory, blank=True)
    slug = models.CharField(max_length=255, unique=True)

    email_template = "df_notifications/base_email.html"
    slack_template = "df_notifications/base_slack.html"

    def get_device_queryset(self):
        return UserDevice.objects.all()

    def send(
        self,
        users=None,
        context=None,
        attachments=None,
    ):
        users = users or []
        attachments = attachments or []
        """
        :param users: users to be notified
        :param context: context
        :param attachments: attachments for email
        :return: history object
        """

        _context = Context(context)

        subject = Template("".join(self.subject.splitlines())).render(_context)

        data = json.loads(Template(self.data).render(_context)) if self.data else {}
        body = Template(self.body).render(_context)

        body_html = Template(
            (
                f'{{% extends "{self.email_template}"%}}{{% block body %}}'
                f"{self.body_html}{{% endblock %}}"
            )
            if self.channel == NotificationChannels.EMAIL
            else self.body_html
        ).render(_context)

        if self.channel == NotificationChannels.EMAIL:
            to_emails = [user.email for user in users if user.email]
            msg = EmailMultiAlternatives(subject=subject, to=to_emails, body=body)
            msg.attach_alternative(body_html, "text/html")
            for attachment in attachments:
                msg.attach(**attachment)
            msg.send()
        elif self.channel == NotificationChannels.PUSH:
            devices = self.get_device_queryset().filter(
                user__in=users,
            )
            devices.send_message(
                Message(
                    notification=Notification(
                        title=subject,
                        body=body,
                    ),
                    data=data,
                ),
            )
        elif self.channel == NotificationChannels.SLACK:
            slack_message(
                self.slack_template,
                context=context,
                attachments=[{"text": body, "title": subject}],
            )
        elif self.channel == NotificationChannels.WEBHOOK:
            requests.post(subject, data=body, json=data)
        elif self.channel == NotificationChannels.CONSOLE:
            logging.info(f"Notification: {subject}")
        else:
            raise NotImplementedError

        history = NotificationHistory.objects.create(
            channel=self.channel,
            data=data,
            body=body,
            subject=subject,
            body_html=body_html,
        )
        history.users.set(users)
        self.history.add(history)
        return history

    def save(self, *args, **kwargs):
        self.data = json.dumps(json.loads(self.data))
        super().save(*args, **kwargs)

    def send_async(self, users, instance, context=None):
        send_notification_async.delay(
            self.pk,
            [str(user.pk) for user in users],
            instance._meta.label_lower,
            str(instance.pk),
            additional_context=context,
        )


@app.task
def send_notification_async(
    template_pk, user_pks, model_name, model_pk, additional_context=None
):
    additional_context = additional_context or {}
    User = get_user_model()
    template: NotificationTemplate = NotificationTemplate.objects.get(pk=template_pk)
    Model = apps.get_model(model_name)
    instance = Model.objects.get(pk=model_pk)
    users = User.objects.filter(pk__in=user_pks)

    template.send(users, {"instance": instance, **additional_context})


class UserDevice(AbstractFCMDevice):
    class Meta:
        verbose_name = _("User device")
        verbose_name_plural = _("User devices")



from .models import NotificationHistory
from .models import NotificationTemplate
from .models import UserDevice
from django.contrib import admin
from fcm_django.admin import DeviceAdmin
from fcm_django.models import FCMDevice


@admin.register(NotificationHistory)
class NotificationHistoryAdmin(admin.ModelAdmin):
    def get_users(self, obj):
        return ",".join(obj.users.values_list("email", flat=True))

    get_users.short_description = "users"

    date_hierarchy = "timestamp"
    search_fields = ("users__email",)
    list_display = (
        "timestamp",
        "channel",
        "get_users",
        "subject",
    )


admin.site.unregister(FCMDevice)


@admin.register(UserDevice)
class UserDeviceAdmin(DeviceAdmin):
    pass


@admin.register(NotificationTemplate)
class NotificationTemplateAdmin(admin.ModelAdmin):
    search_fields = ("slug",)
    list_display = ("slug", "channel", "subject")



from df_notifications.models import UserDevice
from fcm_django.api.rest_framework import FCMDeviceSerializer


class UserDeviceSerializer(FCMDeviceSerializer):
    class Meta(FCMDeviceSerializer.Meta):
        model = UserDevice
        fields = FCMDeviceSerializer.Meta.fields



from df_notifications.models import NotificationTemplate
from django.db.models.signals import post_save
from django.db.models.signals import pre_save

import functools


def save_previous_instance(sender, instance, **kwargs):
    if instance.pk:
        instance._pre_save_instance = sender.objects.get(pk=instance.pk)
    else:
        instance._pre_save_instance = None


def register_notification(model, slugs):
    def decorator(func):
        templates = NotificationTemplate.objects.filter(slug__in=slugs)

        pre_save.connect(save_previous_instance, model, weak=False)

        @functools.wraps(func)
        def wrapper(sender, instance, **kwargs):
            for template in templates:
                func(
                    prev=instance._pre_save_instance,
                    next=instance,
                    template=template,
                    **kwargs
                )

        post_save.connect(wrapper, model, weak=False)

    return decorator
